from django.core.paginator import Paginator


DEFAULT_PAGE_KEY = "p"


def paginate(request, items, page_key=DEFAULT_PAGE_KEY, per_page=20):
    paginator = Paginator(items, per_page)
    page = paginator.get_page(request.GET.get(page_key))
    return paginator, page



from django.apps import AppConfig


class WagtailMediaAppConfig(AppConfig):
    default_auto_field = "django.db.models.AutoField"
    name = "wagtailmedia"
    label = "wagtailmedia"
    verbose_name = "Wagtail media"

    def ready(self):
        from wagtailmedia.signal_handlers import register_signal_handlers

        register_signal_handlers()



from __future__ import absolute_import, unicode_literals

from wagtail.admin.edit_handlers import BaseChooserPanel

from wagtailmedia.widgets import AdminAudioChooser, AdminMediaChooser, AdminVideoChooser


class MediaChooserPanel(BaseChooserPanel):
    object_type_name = "media"

    def __init__(self, field_name, media_type=None, *args, **kwargs):
        super().__init__(field_name=field_name, *args, **kwargs)

        self.media_type = media_type

    def clone_kwargs(self):
        kwargs = super().clone_kwargs()
        kwargs.update(media_type=self.media_type)
        return kwargs

    def widget_overrides(self):
        if self.media_type == "audio":
            widget_class = AdminAudioChooser
        elif self.media_type == "video":
            widget_class = AdminVideoChooser
        else:
            widget_class = AdminMediaChooser

        return {self.field_name: widget_class}



from wagtail.core.permission_policies.collections import (
    CollectionOwnershipPermissionPolicy,
)

from wagtailmedia.models import Media, get_media_model


permission_policy = CollectionOwnershipPermissionPolicy(
    get_media_model(), auth_model=Media, owner_field_name="uploaded_by_user"
)



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-04-20 12:32
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import taggit.managers

from wagtail.core import models as wagtail_models
from wagtail.search import index


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("taggit", "0002_auto_20150616_2121"),
        ("wagtailcore", "0028_merge"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="Media",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("title", models.CharField(max_length=255, verbose_name="title")),
                ("file", models.FileField(upload_to="media", verbose_name="file")),
                (
                    "type",
                    models.CharField(
                        choices=[("audio", "Audio file"), ("video", "Video file")],
                        max_length=255,
                    ),
                ),
                (
                    "duration",
                    models.PositiveIntegerField(
                        help_text="Duration in seconds", verbose_name="duration"
                    ),
                ),
                (
                    "width",
                    models.PositiveIntegerField(
                        blank=True, null=True, verbose_name="width"
                    ),
                ),
                (
                    "height",
                    models.PositiveIntegerField(
                        blank=True, null=True, verbose_name="height"
                    ),
                ),
                (
                    "thumbnail",
                    models.FileField(
                        blank=True,
                        upload_to="media_thumbnails",
                        verbose_name="thumbnail",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(auto_now_add=True, verbose_name="created at"),
                ),
                (
                    "collection",
                    models.ForeignKey(
                        default=wagtail_models.get_root_collection_id,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="+",
                        to="wagtailcore.Collection",
                        verbose_name="collection",
                    ),
                ),
                (
                    "tags",
                    taggit.managers.TaggableManager(
                        blank=True,
                        help_text=None,
                        through="taggit.TaggedItem",
                        to="taggit.Tag",
                        verbose_name="tags",
                    ),
                ),
                (
                    "uploaded_by_user",
                    models.ForeignKey(
                        blank=True,
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to=settings.AUTH_USER_MODEL,
                        verbose_name="uploaded by user",
                    ),
                ),
            ],
            options={
                "abstract": False,
                "verbose_name": "media",
            },
            bases=(models.Model, index.Indexed),
        ),
    ]



from __future__ import unicode_literals

from django import forms
from django.forms.models import modelform_factory
from django.utils.module_loading import import_string
from django.utils.translation import gettext_lazy as _

from wagtail.admin import widgets
from wagtail.admin.forms.collections import (
    BaseCollectionMemberForm,
    collection_member_permission_formset_factory,
)

from wagtailmedia.models import Media
from wagtailmedia.permissions import permission_policy as media_permission_policy
from wagtailmedia.settings import wagtailmedia_settings


class BaseMediaForm(BaseCollectionMemberForm):
    class Meta:
        widgets = {
            "tags": widgets.AdminTagWidget,
            "file": forms.FileInput,
            "thumbnail": forms.ClearableFileInput,
        }

    permission_policy = media_permission_policy

    def __init__(self, *args, **kwargs):
        super(BaseMediaForm, self).__init__(*args, **kwargs)

        if self.instance.type == "audio":
            for name in ("width", "height"):
                # these fields might be editable=False so verify before accessing
                if name in self.fields:
                    del self.fields[name]


def get_media_base_form():
    base_form_override = wagtailmedia_settings.MEDIA_FORM_BASE
    if base_form_override:
        base_form = import_string(base_form_override)
    else:
        base_form = BaseMediaForm
    return base_form


media_base_form = get_media_base_form()


def get_media_form(model):
    fields = model.admin_form_fields
    if "collection" not in fields:
        # force addition of the 'collection' field, because leaving it out can
        # cause dubious results when multiple collections exist (e.g adding the
        # media to the root collection where the user may not have permission) -
        # and when only one collection exists, it will get hidden anyway.
        fields = list(fields) + ["collection"]

    return modelform_factory(
        model,
        form=media_base_form,
        fields=fields,
    )


GroupMediaPermissionFormSet = collection_member_permission_formset_factory(
    Media,
    [
        ("add_media", _("Add"), _("Add/edit media you own")),
        ("change_media", _("Edit"), _("Edit any media")),
    ],
    "wagtailmedia/permissions/includes/media_permissions_formset.html",
)



from django.shortcuts import get_object_or_404, render
from django.urls import reverse

from wagtail.admin.auth import PermissionPolicyChecker
from wagtail.admin.forms.search import SearchForm
from wagtail.admin.modal_workflow import render_modal_workflow
from wagtail.admin.models import popular_tags_for_model
from wagtail.core import hooks
from wagtail.core.models import Collection
from wagtail.search.backends import get_search_backends

from wagtailmedia.forms import get_media_form
from wagtailmedia.models import get_media_model
from wagtailmedia.permissions import permission_policy
from wagtailmedia.utils import paginate


pagination_template = "wagtailadmin/shared/ajax_pagination_nav.html"
permission_checker = PermissionPolicyChecker(permission_policy)


def get_media_json(media):
    """
    helper function: given a media, return the json to pass back to the
    chooser panel
    """

    return {
        "id": media.id,
        "title": media.title,
        "edit_link": reverse("wagtailmedia:edit", args=(media.id,)),
    }


def chooser(request, media_type=None):
    Media = get_media_model()

    media_files = permission_policy.instances_user_has_any_permission_for(
        request.user, ["change", "delete"]
    )

    # allow hooks to modify the queryset
    for hook in hooks.get_hooks("construct_media_chooser_queryset"):
        media_files = hook(media_files, request)

    if permission_policy.user_has_permission(request.user, "add"):
        MediaForm = get_media_form(Media)
        media_audio = Media(uploaded_by_user=request.user, type="audio")
        media_video = Media(uploaded_by_user=request.user, type="video")

        uploadforms = {
            "audio": MediaForm(
                user=request.user, prefix="media-chooser-upload", instance=media_audio
            ),
            "video": MediaForm(
                user=request.user, prefix="media-chooser-upload", instance=media_video
            ),
        }

        if media_type:
            uploadforms = {media_type: uploadforms[media_type]}
    else:
        uploadforms = {}

    if media_type:
        media_files = media_files.filter(type=media_type)

    if (
        "q" in request.GET
        or "p" in request.GET
        or "tag" in request.GET
        or "collection_id" in request.GET
    ):
        collection_id = request.GET.get("collection_id")
        if collection_id:
            media_files = media_files.filter(collection=collection_id)

        searchform = SearchForm(request.GET)
        if searchform.is_valid():
            q = searchform.cleaned_data["q"]

            media_files = media_files.search(q)
            is_searching = True
        else:
            media_files = media_files.order_by("-created_at")
            is_searching = False
            q = None

            tag_name = request.GET.get("tag")
            if tag_name:
                media_files = media_files.filter(tags__name=tag_name)

        # Pagination
        paginator, media_files = paginate(request, media_files, per_page=10)

        return render(
            request,
            "wagtailmedia/chooser/results.html",
            {
                "media_files": media_files,
                "query_string": q,
                "is_searching": is_searching,
                "pagination_template": pagination_template,
                "media_type": media_type,
            },
        )
    else:
        searchform = SearchForm()

        collections = Collection.objects.all()
        if len(collections) < 2:
            collections = None

        media_files = media_files.order_by("-created_at")
        paginator, media_files = paginate(request, media_files, per_page=10)

    return render_modal_workflow(
        request,
        "wagtailmedia/chooser/chooser.html",
        None,
        {
            "media_files": media_files,
            "searchform": searchform,
            "collections": collections,
            "uploadforms": uploadforms,
            "is_searching": False,
            "pagination_template": pagination_template,
            "popular_tags": popular_tags_for_model(Media),
            "media_type": media_type,
        },
        json_data={
            "step": "chooser",
            "error_label": "Server Error",
            "error_message": "Report this error to your webmaster with the following information:",
            "tag_autocomplete_url": reverse("wagtailadmin_tag_autocomplete"),
        },
    )


def media_chosen(request, media_id):
    media = get_object_or_404(get_media_model(), id=media_id)

    return render_modal_workflow(
        request,
        None,
        None,
        None,
        json_data={"step": "media_chosen", "result": get_media_json(media)},
    )


@permission_checker.require("add")
def chooser_upload(request, media_type):
    upload_forms = {}

    if (
        permission_policy.user_has_permission(request.user, "add")
        and request.method == "POST"
    ):
        Media = get_media_model()
        MediaForm = get_media_form(Media)

        media = Media(uploaded_by_user=request.user, type=media_type)
        uploading_form = MediaForm(
            request.POST,
            request.FILES,
            instance=media,
            user=request.user,
            prefix="media-chooser-upload",
        )
        if uploading_form.is_valid():
            uploading_form.save()

            # Reindex the media entry to make sure all tags are indexed
            for backend in get_search_backends():
                backend.add(media)

            return render_modal_workflow(
                request,
                None,
                None,
                None,
                json_data={"step": "media_chosen", "result": get_media_json(media)},
            )

        if media_type == "audio":
            video = Media(uploaded_by_user=request.user, type="video")
            video_form = MediaForm(
                instance=video, user=request.user, prefix="media-chooser-upload"
            )
            upload_forms = {"audio": uploading_form, "video": video_form}
        else:
            audio = Media(uploaded_by_user=request.user, type="audio")
            audio_form = MediaForm(
                instance=audio, user=request.user, prefix="media-chooser-upload"
            )
            upload_forms = {"audio": audio_form, "video": uploading_form}

    media_files = permission_policy.instances_user_has_any_permission_for(
        request.user, ["change", "delete"]
    )

    # allow hooks to modify the queryset
    for hook in hooks.get_hooks("construct_media_chooser_queryset"):
        media_files = hook(media_files, request)

    searchform = SearchForm()

    collections = Collection.objects.all()
    if len(collections) < 2:
        collections = None

    media_files = media_files.order_by("-created_at")
    paginator, media_files = paginate(request, media_files, per_page=10)

    context = {
        "media_files": media_files,
        "searchform": searchform,
        "collections": collections,
        "uploadforms": upload_forms,
        "is_searching": False,
        "pagination_template": pagination_template,
        "media_type": media_type,
    }
    return render_modal_workflow(
        request,
        "wagtailmedia/chooser/chooser.html",
        None,
        context,
        json_data={"step": "chooser"},
    )



#!/usr/bin/env python

import io

from setuptools import find_packages, setup

from src.wagtailmedia import __version__


# Testing dependencies
testing_extras = [
    # Required for running the tests
    "mock>=1.0.0",
    # For coverage and PEP8 linting
    "coverage>=3.7.0",
    "tox==3.23.1",
]

with io.open("README.md", encoding="utf-8") as readme_file:
    long_description = readme_file.read()

setup(
    name="wagtailmedia",
    version=__version__,
    description="A module for Wagtail that provides functionality "
    "similar to wagtail.documents module, but for audio and video files.",
    author="Mikalai Radchuk",
    author_email="hello@torchbox.com",
    maintainer="Dan Braghis",
    maintainer_email="dan.braghis@torchbox.com",
    project_urls={
        "Changelog": "https://github.com/torchbox/wagtailmedia/blob/main/CHANGELOG.md",
    },
    url="https://github.com/torchbox/wagtailmedia",
    packages=find_packages(where="src"),
    package_dir={"": "src"},
    include_package_data=True,
    license="BSD",
    long_description=long_description,
    long_description_content_type="text/markdown",
    keywords=["wagtail", "django", "media"],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Framework :: Django",
        "Framework :: Django :: 2.2",
        "Framework :: Django :: 3.1",
        "Framework :: Django :: 3.2",
        "Framework :: Wagtail",
        "Framework :: Wagtail :: 2",
        "Topic :: Internet :: WWW/HTTP :: Site Management",
    ],
    install_requires=[
        "wagtail>=2.11",
    ],
    extras_require={
        "testing": testing_extras,
    },
    zip_safe=False,
)



from django.urls import path, re_path

from wagtailmedia.views import chooser, media


urlpatterns = [
    path("", media.index, name="index"),
    re_path(r"^(?P<media_type>audio|video|media)/add/$", media.add, name="add"),
    path("edit/<int:media_id>/", media.edit, name="edit"),
    path("delete/<int:media_id>/", media.delete, name="delete"),
    path("chooser/", chooser.chooser, name="chooser"),
    re_path(
        r"chooser/(?P<media_type>audio|video)/", chooser.chooser, name="chooser_typed"
    ),
    path("chooser/<int:media_id>/", chooser.media_chosen, name="media_chosen"),
    re_path(
        r"^(?P<media_type>audio|video)/chooser/upload/$",
        chooser.chooser_upload,
        name="chooser_upload",
    ),
    path("usage/<int:media_id>/", media.usage, name="media_usage"),
]



from __future__ import unicode_literals

from django.forms import ModelChoiceField
from django.forms.utils import flatatt
from django.utils.functional import cached_property
from django.utils.html import format_html, format_html_join
from django.utils.translation import gettext_lazy as _

from wagtail.core.blocks import ChooserBlock


class AbstractMediaChooserBlock(ChooserBlock):
    def __init__(
        self, required=True, help_text=None, validators=(), media_type=None, **kwargs
    ):
        super().__init__(
            required=required, help_text=help_text, validators=validators, **kwargs
        )
        self.media_type = media_type

    @cached_property
    def target_model(self):
        from wagtailmedia.models import get_media_model

        return get_media_model()

    @cached_property
    def field(self):
        if not self.media_type:
            return super().field

        return ModelChoiceField(
            queryset=self.target_model.objects.filter(type=self.media_type),
            widget=self.widget,
            required=self._required,
            validators=self._validators,
            help_text=self._help_text,
        )

    @cached_property
    def widget(self):
        from wagtailmedia.widgets import AdminMediaChooser

        return AdminMediaChooser()

    def get_form_state(self, value):
        return self.widget.get_value_data(value)

    def render_basic(self, value, context=None):
        raise NotImplementedError(
            "You need to implement %s.render_basic" % self.__class__.__name__
        )


class AudioChooserBlock(AbstractMediaChooserBlock):
    def __init__(self, required=True, help_text=None, validators=(), **kwargs):
        super().__init__(
            required=required,
            help_text=help_text,
            validators=validators,
            media_type="audio",
            **kwargs,
        )

    @cached_property
    def widget(self):
        from wagtailmedia.widgets import AdminAudioChooser

        return AdminAudioChooser()

    def render_basic(self, value, context=None):
        if not value:
            return ""

        if value.type != self.media_type:
            return ""

        return format_html(
            "<audio controls>\n{sources}\n<p>{fallback}</p>\n</audio>",
            sources=format_html_join(
                "\n", "<source{0}>", [[flatatt(s)] for s in value.sources]
            ),
            fallback=_("Your browser does not support the audio element."),
        )


class VideoChooserBlock(AbstractMediaChooserBlock):
    def __init__(self, required=True, help_text=None, validators=(), **kwargs):
        super().__init__(
            required=required,
            help_text=help_text,
            validators=validators,
            media_type="video",
            **kwargs,
        )

    @cached_property
    def widget(self):
        from wagtailmedia.widgets import AdminVideoChooser

        return AdminVideoChooser()

    def render_basic(self, value, context=None):
        if not value:
            return ""

        if value.type != self.media_type:
            return ""

        return format_html(
            "<video controls>\n{sources}\n<p>{fallback}</p>\n</video>",
            sources=format_html_join(
                "\n", "<source{0}>", [[flatatt(s)] for s in value.sources]
            ),
            fallback=_("Your browser does not support the video element."),
        )
